@@ -46,6 +46,7 @@ from pyroute2.netlink.rtnl import RTM_SETLINK
 from pyroute2.netlink.rtnl import RTM_GETNEIGHTBL
 from pyroute2.netlink.rtnl import RTM_GETNSID
 from pyroute2.netlink.rtnl import RTM_NEWNETNS
+from pyroute2.netlink.rtnl import RTM_GETSTATS
 from pyroute2.netlink.rtnl import TC_H_ROOT
 from pyroute2.netlink.rtnl import rt_type
 from pyroute2.netlink.rtnl import rt_scope
@@ -64,6 +65,7 @@ from pyroute2.netlink.rtnl.fibmsg import fibmsg
 from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
 from pyroute2.netlink.rtnl.ifinfmsg import IFF_NOARP
 from pyroute2.netlink.rtnl.ifaddrmsg import ifaddrmsg
+from pyroute2.netlink.rtnl.ifstatsmsg import ifstatsmsg
 from pyroute2.netlink.rtnl.iprsocket import IPRSocket
 from pyroute2.netlink.rtnl.iprsocket import IPBatchSocket
 from pyroute2.netlink.rtnl.riprsocket import RawIPRSocket
@@ -171,6 +173,8 @@ class RTNL_API(object):
                        self.get_addr,
                        self.get_neighbours,
                        self.get_routes,
+                       self.get_vlans,
+                       partial(self.fdb, 'dump'),
                        partial(self.get_rules, family=AF_INET),
                        partial(self.get_rules, family=AF_INET6)):
             for msg in method():
@@ -412,6 +416,7 @@ class RTNL_API(object):
             if nsfd > 0:
                 os.close(nsfd)
         item['header']['type'] = RTM_NEWNETNS
+        item['header']['target'] = self.target
         item['event'] = 'RTM_NEWNETNS'
         return item
 
@@ -1644,6 +1649,14 @@ class RTNL_API(object):
                      dst="10.0.0.0/24",
                      type="blackhole")
 
+        Create a route with metrics::
+
+            ip.route('add',
+                     dst='172.16.0.0/24',
+                     gateway='10.0.0.10',
+                     metrics={'mtu': 1400,
+                              'hoplimit': 16})
+
         Multipath route::
 
             ip.route("add",
@@ -1661,6 +1674,16 @@ class RTNL_API(object):
                      encap={"type": "mpls",
                             "labels": "200/300"})
 
+        Create MPLS route: push label::
+
+            # $ sudo modprobe mpls_router
+            # $ sudo sysctl net.mpls.platform_labels=1024
+            ip.route('add',
+                     family=AF_MPLS,
+                     oif=idx,
+                     dst=0x200,
+                     newdst=[0x200, 0x300])
+
         MPLS multipath::
 
             idx = ip.link_lookup(ifname='eth0')[0]
@@ -1838,6 +1861,8 @@ class RTNL_API(object):
 
         for key in kwarg:
             nla = rtmsg.name2nla(key)
+            if nla == 'RTA_DST' and not kwarg[key]:
+                continue
             if kwarg[key] is not None:
                 msg['attrs'].append([nla, kwarg[key]])
                 # fix IP family, if needed
@@ -1987,6 +2012,34 @@ class RTNL_API(object):
             ret = tuple(ret)
 
         return ret
+
+    def stats(self, command, **kwarg):
+        '''
+        Stats prototype.
+        '''
+        if (command == 'dump') and ('match' not in kwarg):
+            match = kwarg
+        else:
+            match = kwarg.pop('match', None)
+
+        commands = {'dump': (RTM_GETSTATS, NLM_F_REQUEST | NLM_F_DUMP),
+                    'get': (RTM_GETSTATS, NLM_F_REQUEST | NLM_F_ACK)}
+        command, flags = commands.get(command, command)
+
+        msg = ifstatsmsg()
+        msg['filter_mask'] = kwarg.get('filter_mask', 31)
+        msg['ifindex'] = kwarg.get('ifindex', 0)
+
+        ret = self.nlm_request(msg,
+                               msg_type=command,
+                               msg_flags=flags)
+        if match is not None:
+            ret = self._match(match, ret)
+
+        if not (command == RTM_GETSTATS and self.nlm_generator):
+            ret = tuple(ret)
+
+        return ret
     # 8<---------------------------------------------------------------
 
 
