@@ -1,6 +1,4 @@
 '''
-RTNL sources
-============
 
 Local RTNL
 ----------
@@ -70,6 +68,7 @@ In some more extended form::
 
 See also: :ref:`remote`
 '''
+import sys
 import time
 import errno
 import socket
@@ -77,16 +76,20 @@ import struct
 import threading
 from pyroute2 import IPRoute
 from pyroute2 import RemoteIPRoute
-from pyroute2.netns.nslink import NetNS
-from pyroute2.netns.manager import NetNSManager
-from pyroute2.ndb.events import (SyncStart,
-                                 SchemaReadLock,
-                                 SchemaReadUnlock,
-                                 ShutdownException,
-                                 MarkFailed,
+from pyroute2.ndb.events import (ShutdownException,
                                  State)
+from pyroute2.ndb.messages import (cmsg_event,
+                                   cmsg_failed,
+                                   cmsg_sstart)
 from pyroute2.netlink.nlsocket import NetlinkMixin
 from pyroute2.netlink.exceptions import NetlinkError
+if sys.platform.startswith('linux'):
+    from pyroute2 import netns
+    from pyroute2.netns.nslink import NetNS
+    from pyroute2.netns.manager import NetNSManager
+else:
+    NetNS = None
+    NetNSManager = None
 
 SOURCE_FAIL_PAUSE = 5
 
@@ -116,12 +119,10 @@ class Source(dict):
         self.ndb = ndb
         self.evq = self.ndb._event_queue
         # the target id -- just in case
-        self.target = spec.pop('target')
+        self.target = spec['target']
         self.kind = spec.pop('kind', 'local')
         self.persistent = spec.pop('persistent', True)
         self.event = spec.pop('event')
-        if not self.event:
-            self.event = SyncStart()
         # RTNL API
         self.nl_prime = self.vmap[self.kind]
         self.nl_kwarg = spec
@@ -129,7 +130,7 @@ class Source(dict):
         self.shutdown = threading.Event()
         self.started = threading.Event()
         self.lock = threading.RLock()
-        self.shutdown_lock = threading.Lock()
+        self.shutdown_lock = threading.RLock()
         self.started.clear()
         self.log = ndb.log.channel('sources.%s' % self.target)
         self.state = State(log=self.log)
@@ -165,6 +166,7 @@ class Source(dict):
         if 'netns' in spec:
             defaults['kind'] = 'netns'
             defaults['target'] = spec['netns']
+            ret['netns'] = netns._get_netnspath(spec['netns'])
         for key in defaults:
             if key not in ret:
                 ret[key] = defaults[key]
@@ -200,7 +202,6 @@ class Source(dict):
                         struct.error):
                     raise
                 except Exception as e:
-                    print(type(e))
                     # probably the source is restarting
                     self.log.debug('source api error: %s' % e)
                     time.sleep(1)
@@ -243,20 +244,22 @@ class Source(dict):
                     self.ndb.schema.allow_read(False)
                     try:
                         self.ndb.schema.flush(self.target)
-                        self.evq.put((self.target, self.nl.dump()))
+                        self.evq.put(self.nl.dump())
                     finally:
                         self.ndb.schema.allow_read(True)
                     self.started.set()
                     self.shutdown.clear()
                     self.state.set('running')
                     if self.event is not None:
-                        self.evq.put((self.target, (self.event, )))
+                        self.evq.put((cmsg_event(self.target, self.event), ))
+                    else:
+                        self.evq.put((cmsg_sstart(self.target), ))
                 except Exception as e:
                     self.started.set()
                     self.state.set('failed')
                     self.log.error('source error: %s %s' % (type(e), e))
                     try:
-                        self.evq.put((self.target, (MarkFailed(), )))
+                        self.evq.put((cmsg_failed(self.target), ))
                     except ShutdownException:
                         stop = True
                         break
@@ -292,7 +295,7 @@ class Source(dict):
 
                 self.ndb.schema._allow_write.wait()
                 try:
-                    self.evq.put((self.target, msg))
+                    self.evq.put(msg)
                 except ShutdownException:
                     stop = True
                     break
@@ -312,7 +315,7 @@ class Source(dict):
     def sync(self):
         self.log.debug('sync')
         sync = threading.Event()
-        self.evq.put((self.target, (sync, )))
+        self.evq.put((cmsg_event(self.target, sync), ))
         sync.wait()
 
     def start(self):
@@ -351,16 +354,19 @@ class Source(dict):
 
     def restart(self, reason='unknown'):
         with self.lock:
-            if not self.shutdown.is_set():
+            with self.shutdown_lock:
                 self.log.debug('restarting the source, reason <%s>' % (reason))
-                self.evq.put((self.target, (SchemaReadLock(), )))
+                self.started.clear()
+                self.ndb.schema.allow_read(False)
                 try:
                     self.close()
                     if self.th:
                         self.th.join()
+                    self.shutdown.clear()
                     self.start()
                 finally:
-                    self.evq.put((self.target, (SchemaReadUnlock(), )))
+                    self.ndb.schema.allow_read(True)
+        self.started.wait()
 
     def __enter__(self):
         return self
