@@ -178,6 +178,7 @@ There are several pre-defined NLA types, that you will get with
     - `ip6addr` -- only IPv6 address type
     - `target` -- a univeral target (IPv4, IPv6, MPLS)
     - `l2addr` -- MAC address
+    - `lladdr` -- link layer address (MAC, IPv4, IPv6)
     - `hex` -- hex dump as a string -- useful for debugging
     - `cdata` -- a binary data
     - `string` -- UTF-8 string
@@ -401,6 +402,7 @@ from socket import inet_ntop
 from socket import AF_INET
 from socket import AF_INET6
 from socket import AF_UNSPEC
+from collections import OrderedDict
 from pyroute2.common import AF_MPLS
 from pyroute2.common import hexdump
 from pyroute2.common import basestring
@@ -575,6 +577,65 @@ cache_hdr = {}
 cache_jit = {}
 
 
+class SQLSchema(object):
+
+    def __init__(self, cls):
+        ret = []
+        for field in cls.fields:
+            if field[0][0] != '_':
+                ret.append(((field[0], ),
+                            ' '.join(('BIGINT',
+                                      cls.sql_constraints.get(field[0], '')))))
+        for nla in cls.nla_map:
+            if isinstance(nla[0], basestring):
+                nla_name = nla[0]
+                nla_type = nla[1]
+            else:
+                nla_name = nla[1]
+                nla_type = nla[2]
+            nla_type = getattr(cls, nla_type, None)
+            sql_type = getattr(nla_type, 'sql_type', None)
+            if sql_type:
+                sql_type = ' '.join((sql_type,
+                                     cls.sql_constraints.get(nla_name, '')))
+                ret.append(((nla_name, ), sql_type))
+
+        for (fname, ftype) in cls.sql_extra_fields:
+            if isinstance(fname, basestring):
+                fname = (fname, )
+            ret.append((fname, ftype))
+
+        for (dcls, prefix) in cls.sql_extend:
+            for fname, ftype in dcls.sql_schema():
+                ret.append(((prefix, ) + fname, ftype))
+
+        self.spec = ret
+        self.index = []
+        self.foreign_keys = []
+
+    def unique_index(self, *index):
+        self.index = index
+        return self
+
+    def foreign_key(self, parent, fields, parent_fields):
+        self.foreign_keys.append({'fields': fields,
+                                  'parent_fields': parent_fields,
+                                  'parent': parent})
+        return self
+
+    def push(self, *spec):
+        f_type = spec[-1]
+        f_name = spec[:-1]
+        self.spec.append((f_name, f_type))
+        return self
+
+    def __iter__(self):
+        return iter(self.spec)
+
+    def as_dict(self):
+        return OrderedDict(self.spec)
+
+
 class nlmsg_base(dict):
     '''
     Netlink base class. You do not need to inherit it directly, unless
@@ -606,6 +667,8 @@ class nlmsg_base(dict):
     __compiled_ft = False
     __t_nla_map = None
     __r_nla_map = None
+    # schema
+    __schema = None
 
     __slots__ = (
         "_buf",
@@ -671,36 +734,7 @@ class nlmsg_base(dict):
 
     @classmethod
     def sql_schema(cls):
-        ret = []
-        for field in cls.fields:
-            if field[0][0] != '_':
-                ret.append(((field[0], ),
-                            ' '.join(('BIGINT',
-                                      cls.sql_constraints.get(field[0], '')))))
-        for nla in cls.nla_map:
-            if isinstance(nla[0], basestring):
-                nla_name = nla[0]
-                nla_type = nla[1]
-            else:
-                nla_name = nla[1]
-                nla_type = nla[2]
-            nla_type = getattr(cls, nla_type, None)
-            sql_type = getattr(nla_type, 'sql_type', None)
-            if sql_type:
-                sql_type = ' '.join((sql_type,
-                                     cls.sql_constraints.get(nla_name, '')))
-                ret.append(((nla_name, ), sql_type))
-
-        for (fname, ftype) in cls.sql_extra_fields:
-            if isinstance(fname, basestring):
-                fname = (fname, )
-            ret.append((fname, ftype))
-
-        for (dcls, prefix) in cls.sql_extend:
-            for fname, ftype in dcls.sql_schema():
-                ret.append(((prefix, ) + fname, ftype))
-
-        return ret
+        return SQLSchema(cls)
 
     @property
     def buf(self):
@@ -1017,7 +1051,8 @@ class nlmsg_base(dict):
                                   offset=offset,
                                   parent=self)
                 cell._nla_array = False
-                cell['header']['type'] = self.header_type or header_type
+                cell['header']['type'] = self.header_type or \
+                    (header_type | self._nla_flags)
                 header_type += 1
 
                 if cell.cell_header is not None:
@@ -1594,14 +1629,14 @@ class nlmsg_atoms(nlmsg_base):
     class uint32(nla_base):
 
         __slots__ = ()
-        sql_type = 'INTEGER'
+        sql_type = 'BIGINT'
 
         fields = [('value', 'I')]
 
     class uint64(nla_base):
 
         __slots__ = ()
-        sql_type = 'INTEGER'
+        sql_type = 'BIGINT'
 
         fields = [('value', 'Q')]
 
@@ -1622,14 +1657,14 @@ class nlmsg_atoms(nlmsg_base):
     class int32(nla_base):
 
         __slots__ = ()
-        sql_type = 'INTEGER'
+        sql_type = 'BIGINT'
 
         fields = [('value', 'i')]
 
     class int64(nla_base):
 
         __slots__ = ()
-        sql_type = 'INTEGER'
+        sql_type = 'BIGINT'
 
         fields = [('value', 'q')]
 
@@ -1650,14 +1685,14 @@ class nlmsg_atoms(nlmsg_base):
     class be32(nla_base):
 
         __slots__ = ()
-        sql_type = 'INTEGER'
+        sql_type = 'BIGINT'
 
         fields = [('value', '>I')]
 
     class be64(nla_base):
 
         __slots__ = ()
-        sql_type = 'INTEGER'
+        sql_type = 'BIGINT'
 
         fields = [('value', '>Q')]
 
@@ -1824,6 +1859,48 @@ class nlmsg_atoms(nlmsg_base):
             self.value = ':'.join('%02x' % (i) for i in
                                   struct.unpack('BBBBBB', self['value']))
 
+    class lladdr(nla_base):
+        '''
+        Decode link layer address: a MAC, IPv4 or IPv6 address. This type
+        depends on the link layer address length:
+
+        * 6: MAC addr, string: "52:ff:ff:ff:ff:03"
+        * 4: IPv4 addr, string: "127.0.0.1"
+        * 16: IPv6 addr, string: "::1"
+        '''
+
+        __slots__ = ()
+        sql_type = 'TEXT'
+
+        fields = [('value', 's')]
+
+        def encode(self):
+            if ':' in self.value:
+                if len(self.value) == 17 and '::' not in self.value:
+                    self['value'] = struct.pack('BBBBBB',
+                                                *[int(i, 16) for i in
+                                                  self.value.split(':')])
+                else:
+                    self['value'] = inet_pton(AF_INET6, self.value)
+            elif '.' in self.value:
+                self['value'] = inet_pton(AF_INET, self.value)
+            else:
+                raise TypeError('Unsupported value {}'.format(self.value))
+            nla_base.encode(self)
+
+        def decode(self):
+            nla_base.decode(self)
+            if len(self['value']) == 6:
+                self.value = ':'.join('%02x' % (i) for i in
+                                      struct.unpack('BBBBBB', self['value']))
+            elif len(self['value']) == 4:
+                self.value = inet_ntop(AF_INET, self['value'])
+            elif len(self['value']) == 16:
+                self.value = inet_ntop(AF_INET6, self['value'])
+            else:
+                raise TypeError('Unsupported link layer address length {}'
+                                .format(len(self['value'])))
+
     class hex(nla_base):
         '''
         Represent NLA's content with header as hex string.
