@@ -291,7 +291,8 @@ class NetlinkMixin(object):
                  rcvbuf=1048576,
                  all_ns=False,
                  async_qsize=None,
-                 nlm_generator=None):
+                 nlm_generator=None,
+                 target='localhost'):
         #
         # That's a trick. Python 2 is not able to construct
         # sockets from an open FD.
@@ -332,6 +333,7 @@ class NetlinkMixin(object):
         self.pthread = None
         self.closed = False
         self.uname = config.uname
+        self.target = target
         self.capabilities = {'create_bridge': config.kernel > [3, 2, 0],
                              'create_bond': config.kernel > [3, 2, 0],
                              'create_dummy': True,
@@ -384,12 +386,6 @@ class NetlinkMixin(object):
         # Set defaults
         self.post_init()
 
-    def __del__(self):
-        try:
-            self.close()
-        except Exception:
-            pass
-
     def post_init(self):
         pass
 
@@ -801,6 +797,7 @@ class NetlinkMixin(object):
                                 # We've got the data, lock the backlog again
                                 with self.backlog_lock:
                                     for msg in msgs:
+                                        msg['header']['target'] = self.target
                                         msg['header']['stats'] = Stats(current,
                                                                        delta,
                                                                        delay)
@@ -886,7 +883,7 @@ class NetlinkMixin(object):
                     #
                     # It's a long story. Modern kernels for RTM_SET.*
                     # operations always return NLMSG_ERROR(0) == success,
-                    # even not setting NLM_F_MULTY flag on other response
+                    # even not setting NLM_F_MULTI flag on other response
                     # messages and thus w/o any NLMSG_DONE. So, how to detect
                     # the response end? One can not rely on NLMSG_ERROR on
                     # old kernels, but we have to support them too. Ty, we
