@@ -17,13 +17,15 @@ from pyroute2.netlink.rtnl.iw_event import iw_event
 from pyroute2.netlink.rtnl.ifinfmsg.plugins import (bond,
                                                     gtp,
                                                     ipvlan,
+                                                    team,
                                                     tuntap,
                                                     vlan,
                                                     vrf,
                                                     vti,
                                                     vti6,
                                                     vxlan,
-                                                    xfrm)
+                                                    xfrm,
+                                                    ipoib)
 
 log = logging.getLogger(__name__)
 
@@ -213,13 +215,15 @@ data_plugins = {}
 for module in (bond,
                gtp,
                ipvlan,
+               team,
                tuntap,
                vlan,
                vrf,
                vti,
                vti6,
                vxlan,
-               xfrm):
+               xfrm,
+               ipoib):
     name = module.__name__.split('.')[-1]
     data_plugins[name] = getattr(module, name)
 
@@ -515,10 +519,42 @@ class ifinfbase(object):
                        ('IFLA_VF_RSS_QUERY_EN', 'vf_rss_query_en'),
                        ('IFLA_VF_STATS', 'vf_stats'),
                        ('IFLA_VF_TRUST', 'vf_trust'),
-                       ('IFLA_VF_IB_NODE_GUID', 'hex'),
-                       ('IFLA_VF_IB_PORT_GUID', 'hex'),
+                       ('IFLA_VF_IB_NODE_GUID', 'vf_ib_node_guid'),
+                       ('IFLA_VF_IB_PORT_GUID', 'vf_ib_port_guid'),
                        ('IFLA_VF_VLAN_LIST', 'vf_vlist'))
 
+            class vf_ib_node_guid(nla):
+                fields = (('vf', 'I'),
+                          ('ib_node_guid', '32B'))
+
+                def decode(self):
+                    nla.decode(self)
+                    self['ib_node_guid'] = ':'.join([
+                        '%02x' % x for x in self['ib_node_guid'][4:12][::-1]])
+
+                def encode(self):
+                    encoded_guid = self['ib_node_guid'].split(':')[::-1]
+                    self['ib_node_guid'] = (
+                        [0] * 4 + [int(x, 16) for x in encoded_guid] +
+                        [0] * 20)
+                    nla.encode(self)
+
+            class vf_ib_port_guid(nla):
+                fields = (('vf', 'I'),
+                          ('ib_port_guid', '32B'))
+
+                def decode(self):
+                    nla.decode(self)
+                    self['ib_port_guid'] = ':'.join([
+                        '%02x' % x for x in self['ib_port_guid'][4:12][::-1]])
+
+                def encode(self):
+                    encoded_guid = self['ib_port_guid'].split(':')[::-1]
+                    self['ib_port_guid'] = (
+                        [0] * 4 + [int(x, 16) for x in encoded_guid] +
+                        [0] * 20)
+                    nla.encode(self)
+
             class vf_mac(nla):
                 fields = (('vf', 'I'),
                           ('mac', '32B'))
@@ -871,6 +907,7 @@ class ifinfbase(object):
                 nla.encode(self)
 
         class vlan_info(nla):
+            prefix = ''
             fields = (('flags', 'H'),
                       ('vid', 'H'))
 
